<?php
/**
 * SwipeHQ Checkout Payment Module
 *
 * @package paymentMethod
 * @copyright Copyright 2012-2013 OptimizerHQ Ltd.
 * @copyright Portions Copyright 2003-2010 Zen Cart Development Team
 * @copyright Portions Copyright 2003 osCommerce
 * @license http://www.zen-cart.com/license/2_0.txt GNU Public License V2.0
 * @version 1.2.0
 */

/**
 *  ensure dependencies are loaded
 */
  include_once((IS_ADMIN_FLAG === true ? DIR_FS_CATALOG_MODULES : DIR_WS_MODULES) . 'payment/swipehq/swipehq_functions.php');

/**
 * swipehq.php payment module class for SwipeHQ Checkout payment method
 *
 */
class swipehq extends base {
  /**
   * string representing the payment method
   *
   * @var string
   */
  var $code;
  /**
   * $title is the displayed name for this payment method
   *
   * @var string
    */
  var $title;
  /**
   * $description is a soft name for this payment method
   *
   * @var string
    */
  var $description;
  /**
   * $enabled determines whether this module shows or not... in catalog.
   *
   * @var boolean
    */
  var $enabled;
  
  
  /**
    * constructor
    *
    * @param int $swipehq_ipn_id
    * @return swipehq
    */
  function swipehq() {
    global $order, $messageStack;
    
    $this->code = 'swipehq';
    $this->codeVersion = '1.3.0';
    $this->title = MODULE_PAYMENT_SWIPEHQ_TEXT_CATALOG_TITLE; // Payment Module title in Catalog
    $this->description = MODULE_PAYMENT_SWIPEHQ_TEXT_DESCRIPTION;
    $this->sort_order = MODULE_PAYMENT_SWIPEHQ_SORT_ORDER;
    $this->enabled = ((MODULE_PAYMENT_SWIPEHQ_STATUS == 'True') ? true : false);
    
    if ((int)MODULE_PAYMENT_SWIPEHQ_ORDER_STATUS_ID > 0) {
      $this->order_status = MODULE_PAYMENT_SWIPEHQ_ORDER_STATUS_ID;
    }
    
    if (is_object($order)) $this->update_status();
  }
  /**
   * calculate zone matches and flag settings to determine whether this module should display to customers or not
    *
    */
  function update_status() {
    global $order, $db;

    if ( ($this->enabled == true) && ((int)MODULE_PAYMENT_SWIPEHQ_ZONE > 0) ) {
      $check_flag = false;
      $check_query = $db->Execute("select zone_id from " . TABLE_ZONES_TO_GEO_ZONES . " where geo_zone_id = '" . MODULE_PAYMENT_SWIPEHQ_ZONE . "' and zone_country_id = '" . $order->billing['country']['id'] . "' order by zone_id");
      while (!$check_query->EOF) {
        if ($check_query->fields['zone_id'] < 1) {
          $check_flag = true;
          break;
        } elseif ($check_query->fields['zone_id'] == $order->billing['zone_id']) {
          $check_flag = true;
          break;
        }
        $check_query->MoveNext();
      }

      if ($check_flag == false) {
        $this->enabled = false;
      }
    }
  }
  /**
   * JS validation which does error-checking of data-entry if this module is selected for use
   * (Number, Owner, and CVV Lengths)
   *
   * @return string
    */
  function javascript_validation() {
    return false;
  }
  /**
   * Displays payment method name along with Credit Card Information Submission Fields (if any) on the Checkout Payment Page
   *
   * @return array
    */
  function selection() {
    return array('id' => $this->code,
                 'module' => MODULE_PAYMENT_SWIPEHQ_TEXT_CATALOG_LOGO,
                 'icon' => MODULE_PAYMENT_SWIPEHQ_TEXT_CATALOG_LOGO
                 );
  }
  /**
   * Normally evaluates the Credit Card Type for acceptance and the validity of the Credit Card Number & Expiration Date
   * Since swipehq module is not collecting info, it simply skips this step.
   *
   * @return boolean
   */
  function pre_confirmation_check() {
    return false;
  }
  /**
   * Display Credit Card Information on the Checkout Confirmation Page
   * Since none is collected for swipehq before forwarding to swipehq site, this is skipped
   *
   * @return boolean
    */
  function confirmation() {
    return false;
  }
    /**
     * Build the data and actions to process when the "Submit" button is pressed on the order-confirmation screen.
     * This sends the data to the payment gateway for processing.
     * (These are hidden fields on the checkout confirmation page)
     *
     * @return string
      */
    function process_button() { 
        return '';
    }
    
    function after_order_create($order_id) {
    	
    	
        if (!empty($order_id)) {
            global $order, $zco_notifier, $order_totals;
            
            $currency = !empty($order->info['currency']) ? $order->info['currency'] : false;
            
            $acceptedCurrencies = $this->getAcceptedCurrencies();
            if (in_array($currency, $acceptedCurrencies)) {
                
                $price = round($order->info['total']*$order->info['currency_value'], 2);
                
                //get list of ordered products
                $products = '';
                $products_list = $order->products;
                foreach ($products_list as $product) {
                    $products .= $product['qty'] . ' x ' . $product['name'] . ($product != end($products_list) ? '<br />' : '');
                }

                //get product ID using TransactionIdentifier API
                $params = array (
                    'merchant_id'           => MODULE_PAYMENT_SWIPEHQ_MERCHANT_ID,
                    'api_key'               => MODULE_PAYMENT_SWIPEHQ_API_KEY,
                    'td_item'               => 'Order ID' . $order_id,
                    'td_description'        => $products,
                    'td_amount'             => $price,
                    'td_currency'           => $currency,
                    'td_default_quantity'   => 1,
                    'td_user_data'          => $order_id,
                    'td_callback_url' => zen_href_link('swipehq_callback.php', '', 'NONSSL',false,false,true),
                    'td_lpn_url' => zen_href_link('swipehq_lpn.php', '', 'NONSSL',false,false,true)
                );

                $api_url = trim(MODULE_PAYMENT_SWIPEHQ_API_URL, '/');
                
                $response = shq_post_to_url($api_url . '/createTransactionIdentifier.php', $params);
                $response_data = json_decode($response); 

                $payment_page_url = trim(MODULE_PAYMENT_SWIPEHQ_PAYMENT_PAGE_URL, '/');
                
                if ($response_data->response_code == 200 && !empty($response_data->data->identifier)) {
                    
                    $zco_notifier->notify('NOTIFY_CHECKOUT_PROCESS_AFTER_PAYMENT_MODULES_AFTER_ORDER_CREATE');
                    // store the product info to the order
                    $order->create_add_products($order_id);
                    $_SESSION['order_number_created'] = $order_id;
                    $zco_notifier->notify('NOTIFY_CHECKOUT_PROCESS_AFTER_ORDER_CREATE_ADD_PRODUCTS');
                    
                    $_SESSION['swipehq_order_' . $order_id] = serialize($order);
                    
                    //redirect to the payment page
                    header('Location: '.$payment_page_url.'?checkout=true&identifier_id=' . $response_data->data->identifier);
                    exit;
                }
            }else{
            	die('The Swipe payment method does not support currency: '.$currency.'. Swipe supports these currencies: '.join(', ', $acceptedCurrencies).'.');
            }
        }
        echo MODULE_PAYMENT_SWIPEHQ_TEXT_ERROR_MESSAGE;
        exit;
    }
    
    /**
     * Store the CC info to the order and process any results that come back from the payment gateway
     *
     */
    function before_process() {
        return false;
    }
  
    /**
     * Post-processing activities
     * When the order returns from the processor, if PDT was successful, this stores the results in order-status-history and logs data for subsequent reference
     *
     * @return boolean
      */
    function after_process() {
        return true;
    }

    /**
    * Checks referrer
    *
    * @param string $zf_domain
    * @return boolean
     */
    function check_referrer($zf_domain) {
      return true;
    }
  
    /**
     * Used to display error message details
     *
     * @return boolean
     */
    function output_error() {
      return false;
    }  
  
    /**
     * Check to see whether module is installed
     *
     * @return boolean
     */
    function check() {
      global $db;
      if (!isset($this->_check)) {
        $check_query = $db->Execute("select configuration_value from " . TABLE_CONFIGURATION . " where configuration_key = 'MODULE_PAYMENT_SWIPEHQ_STATUS'");
        $this->_check = $check_query->RecordCount();
      }
      return $this->_check;
    }
    
    /**
     * Install the payment module and its configuration settings
     *
     */
    function install() {
      global $db, $messageStack;
      if (defined('MODULE_PAYMENT_SWIPEHQ_STATUS')) {
        $messageStack->add_session('SWIPEHQ module already installed.', 'error');
        zen_redirect(zen_href_link(FILENAME_MODULES, 'set=payment&module=swipehq', 'NONSSL'));
        return 'failed';
      }
      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Enable SwipeHQ Checkout Payment Module<script src=\"../includes/modules/payment/swipehq/jquery-1.10.2.min.js\"></script><script src=\"../includes/modules/payment/swipehq/check_config.js\"></script>', 'MODULE_PAYMENT_SWIPEHQ_STATUS', 'True', 'Do you want to accept SwipeHQ Checkout payments?', '6', '0', 'zen_cfg_select_option(array(\'True\', \'False\'), ', now())");
      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Merchant ID', 'MODULE_PAYMENT_SWIPEHQ_MERCHANT_ID', '', 'Find this in your Swipe Merchant login under Settings -> API Credentials', '6', '0', now())");
      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('API Key', 'MODULE_PAYMENT_SWIPEHQ_API_KEY', '', 'Find this in your Swipe Merchant login under Settings -> API Credentials', '6', '0', now())");
      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('API Url', 'MODULE_PAYMENT_SWIPEHQ_API_URL', '', 'Find this in your Swipe Merchant login under Settings -> API Credentials', '6', '0', now())");
      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Payment Page Url', 'MODULE_PAYMENT_SWIPEHQ_PAYMENT_PAGE_URL', '', 'Find this in your Swipe Merchant login under Settings -> API Credentials', '6', '0', now())");
      
      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, date_added) values ('Test Mode', 'MODULE_PAYMENT_SWIPEHQ_TEST_MODE', 'Off', 'Use test mode for processing orders', '6', '0', 'zen_cfg_select_option(array(\'On\', \'Off\'), ', now())");
      
      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, use_function, set_function, date_added) values ('Payment Zone', 'MODULE_PAYMENT_SWIPEHQ_ZONE', '0', 'If a zone is selected, only enable this payment method for that zone.', '6', '0', 'zen_get_zone_class_title', 'zen_cfg_pull_down_zone_classes(', now())");
      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, date_added) values ('Sort order of display.', 'MODULE_PAYMENT_SWIPEHQ_SORT_ORDER', '0', 'Sort order of display. Lowest is displayed first.', '6', '0', now())");
      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, use_function, date_added) values ('Set Unpaid Order Status', 'MODULE_PAYMENT_SWIPEHQ_ORDER_STATUS_ID', '1', 'Set the status of unpaid orders made with this payment module to this value (recommended: Pending [1]).', '6', '0', 'zen_cfg_pull_down_order_statuses(', 'zen_get_order_status_name', now())");
      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, use_function, date_added) values ('Set Paid Order Status', 'MODULE_PAYMENT_SWIPEHQ_PROCESSING_STATUS_ID', '2', 'Set the status of orders made with this payment module to this value (recommended: Processing [2])', '6', '0', 'zen_cfg_pull_down_order_statuses(', 'zen_get_order_status_name', now())");
      $db->Execute("insert into " . TABLE_CONFIGURATION . " (configuration_title, configuration_key, configuration_value, configuration_description, configuration_group_id, sort_order, set_function, use_function, date_added) values ('Set Test Order Status', 'MODULE_PAYMENT_SWIPEHQ_TEST_STATUS_ID', '1', 'Set the status of orders made with test mode turning on (recommended: Pending [1])', '6', '0', 'zen_cfg_pull_down_order_statuses(', 'zen_get_order_status_name', now())");
    }
    
    /**
     * Remove the module and all its settings
     *
     */
    function remove() {
      global $db;
      $db->Execute("delete from " . TABLE_CONFIGURATION . " where configuration_key in ('" . implode("', '", $this->keys()) . "')");
      // Is it better to do: configuration_key ILIKE 'MODULE_PAYMENT_SWIPEHQ%' ?
      // no, we might as well call keys(), as keys() is required by oscommerce
    }
    
    /**
     * Internal list of configuration keys used for configuration of the module
     *
     * @return array
     */
    function keys() {
      return array(
          'MODULE_PAYMENT_SWIPEHQ_STATUS',
          'MODULE_PAYMENT_SWIPEHQ_MERCHANT_ID',
          'MODULE_PAYMENT_SWIPEHQ_API_KEY',
          'MODULE_PAYMENT_SWIPEHQ_API_URL',
      	  'MODULE_PAYMENT_SWIPEHQ_PAYMENT_PAGE_URL',
          'MODULE_PAYMENT_SWIPEHQ_TEST_MODE',
          'MODULE_PAYMENT_SWIPEHQ_ZONE', 
          'MODULE_PAYMENT_SWIPEHQ_ORDER_STATUS_ID',
          'MODULE_PAYMENT_SWIPEHQ_PROCESSING_STATUS_ID',
          'MODULE_PAYMENT_SWIPEHQ_TEST_STATUS_ID',
          'MODULE_PAYMENT_SWIPEHQ_SORT_ORDER');
    }
    
    /**
     * @return array of string currency code
     */
    public function getAcceptedCurrencies(){
    	$api_url = trim(MODULE_PAYMENT_SWIPEHQ_API_URL, '/');
    	 
    	$params = array(
    			'merchant_id' 	=> MODULE_PAYMENT_SWIPEHQ_MERCHANT_ID,
    			'api_key' 	=> MODULE_PAYMENT_SWIPEHQ_API_KEY,
    	);
    	 
    	$response = shq_post_to_url($api_url.'/fetchCurrencyCodes.php', $params);
    	$response_data = json_decode($response, true);
    
    	$r = $response_data['data'];
    	return $r;
    }

}
